/*
 * ml.h
 *
 *
 * declaration of all shared functions and variables of ML engine
 *
 * by Korbinian Strimmer and Arndt von Haeseler (1995-96)
 * this file belongs to the PUZZLE version 2.5.1 distribution 
 *
 */


/* definitions */

#define MINCAT 1
#define MAXCAT 10
#define MINAB 0.5
#define MAXAB 60.0
#define MINYR 0.1
#define MAXYR 6.0
#define MINGA 0.1
#define MAXGA 6.0
#define MINLA 0.0
#define MAXLA 1.0
#define MAXARC     600.0   /* upper limit on branch length (PAM) = 6.0        */
#define MINARC     0.001   /* lower limit on branch length (PAM) = 0.00001    */
#define EPSILON    0.0001  /* error in branch length (PAM) = 0.000001         */
#define MAXIT      30      /* maximum number of iterates of smoothing         */
#define MINFDIFF   0.00002 /* lower limit on base frequency differences       */
#define MINFREQ    0.0001  /* lower limit on base frequencies = 0.01%         */
#define NUMQBRNCH  5       /* number of branches in a quartet                 */
#define NUMQIBRNCH 1       /* number of internal branches in a quartet        */
#define NUMQSPC    4       /* number of taxa in a quartet                     */


/* type definitions */

typedef struct node {
	struct node *isop;
	struct node *kinp;
	int descen;
	int number;
	double length;
	double lklhdl;
	ivector paths;
	cvector eprob;
	dcube partials; /* partial likelihoods */
	char *label;  /* internal labels */
} Node;

typedef struct tree {
	Node *rootp;
	Node **ebrnchp;        /* list of pointers to external branches           */
	Node **ibrnchp;        /* list of pointers to internal branches           */
	double lklhd;          /* total log likelihood                            */
	dmatrix condlkl;       /* log likelihoods depending on rate and site      */
	double varilkl;
	double rssleast;
} Tree;


/* global variables */

EXTERN cmatrix Identif;    /* taxa names                                      */
EXTERN cmatrix Seqchar;    /* ML sequence data                                */
EXTERN cmatrix Seqpat;     /* ordered site patterns                           */
EXTERN cvector seqchi;
EXTERN cvector seqchj;
EXTERN dcube partiali;
EXTERN dcube partialj;
EXTERN dcube ltprobr;      /* log tpms for all rate categories                */
EXTERN dmatrix Distanmat;  /* matrix with maximum likelihood distances        */
EXTERN dmatrix Evec;       /* Eigenvectors                                    */
EXTERN dmatrix Ievc;       /* Inverse eigenvectors                            */
EXTERN double AlphaBeta;   /* Ts rate / Tv rate                               */
EXTERN double tsmean, yrmean, gamean, lamean;
EXTERN double AlphaYR;     /* Y-Ts rate / R-Ts rate                           */
EXTERN double gaerr;       /* estimated error of rate heterogenity            */
EXTERN double Galpha;      /* rate heterogenity parameter                     */
EXTERN double laerr;       /* estimated error of lambda                       */
EXTERN double lambda;      /* rate correlation parameter                      */
EXTERN double Proportion;  /* for tree drawing                                */
EXTERN double tserr;       /* estimated error of AlphaBeta/2.0                */
EXTERN double yrerr;       /* estimated error of AlphaYR                      */
EXTERN dvector Brnlength;
EXTERN dvector Distanvec;
EXTERN dvector Eval;       /* Eigenvalues of 1 PAM rate matrix                */
EXTERN dvector Evl2;       /* Eigenvalues^2                                   */
EXTERN dvector Freqtpm;    /* base frequencies                                */
EXTERN dvector Rates;      /* rate of each of the categories                  */
EXTERN dmatrix iexp;
EXTERN imatrix Basecomp;   /* base composition of each taxon                  */
EXTERN ivector usedtaxa;   /* list needed in the input treefile procedure     */
EXTERN int chi2fail;       /* flag for chi2 test                              */
EXTERN int Converg;        /* flag for ML convergence                         */
EXTERN int data_optn;      /* type of sequence input data                     */  
EXTERN int Dayhf_optn;     /* Dayhoff model                                   */
EXTERN int HKY_optn;       /* use HKY model                                   */
EXTERN int Jtt_optn;       /* JTT model                                       */
EXTERN int lambda_optim;   /* optimize rate correlation parameter             */
EXTERN int Maxsite;        /* number of ML characters per taxum               */
EXTERN int Maxspc;         /* number of taxa                                  */
EXTERN int mlmode;         /* quartet ML or user defined tree ML              */
EXTERN int mtrev_optn;     /* mtREV model                                     */
EXTERN int nuc_optn;       /* nucleotide (4x4) models                         */
EXTERN int Numbrnch;       /* number of branches of current tree              */
EXTERN int numcats;        /* number of rate categories                       */
EXTERN int Numibrnch;      /* number of internal branches of current tree     */
EXTERN int Numit;          /* number of ML iterations if there is convergence */
EXTERN int Numptrn;        /* number of site patterns                         */
EXTERN int Numspc;         /* number of taxa of current tree                  */
EXTERN int optim_optn;     /* optimize model parameters                       */
EXTERN int rate_optim;     /* optimize rate heterogenity parameter            */
EXTERN int show_optn;      /* show unresolved quartets                        */
EXTERN int SH_optn;        /* SH nucleotide (16x16) model                     */
EXTERN int TN_optn;        /* use TN model                                    */
EXTERN int tpmradix;       /* number of different states                      */
EXTERN int tri_optn;       /* generate characteristic triangle                */
EXTERN ivector Alias;      /* link site -> corresponding site pattern         */
EXTERN ivector Weight;     /* weight of each site pattern                     */
EXTERN Tree *Ctree;        /* pointer to current tree                         */


/* function prototypes of all ml function */

void convfreq(dvector freqemp);
void radixsort(cmatrix seqchar, ivector ali, int maxspc, int maxsite, int *numptrn);
void condenceseq(cmatrix seqchar, ivector ali, cmatrix seqconint, ivector weight, int maxspc, int maxsite, int numptrn);
void evaluateseqs(void);
void elmhes(dmatrix a, ivector ordr, int n);
void eltran(dmatrix a, dmatrix zz, ivector ordr, int n);
void cdiv(double ar, double ai, double br, double bi, double *cr, double *ci);
void hqr2(int n, int low, int hgh, dmatrix h, dmatrix zz, dvector wr, dvector wi);
void onepamratematrix(dmatrix a, dvector forg);
void eigensystem(dvector eval, dmatrix evec);
void luinverse(dmatrix imtrx, int size);
void checkevector(int nn);
void tranprobmat(void);
void tprobmtrx(double arc, dmatrix tpr);
double pairlkl(double arc);
double mldistance(int i, int j);
void initdistan(void);
void computedistan(void);
void productpartials(Node *op);
void partialsinternal(Node *op);
void partialsexternal(Node *op);
void initpartials(Tree *tr);
void comptotlkl(void);
void compvartotlkl(void);
double intlkl(double arc);
void optinternalbranch(Node *op);
double extlkl(double arc);
void optexternalbranch(Node *op);
void finishlkl(Node *op);
void truelkl(Tree *tr);
void aproxlkl(Tree *tr);
void luequation(dmatrix amat, dvector yvec, int size);
void lslength(Tree *tr, dvector distanvec, int numspc, int numibrnch);

void getusertree(FILE *itfp, cvector tr, int maxlen);
Node *internalnode(Tree *tr, char **chpp, int *ninode);
void constructtree(Tree *tr, cvector strtree);
void makeusertree(FILE *itfp);
Tree *new_tree(int maxspc, int numptrn, cmatrix seqconint);
Tree *new_quartet(int numptrn, cmatrix seqconint);
void free_tree(Tree *tr, int taxa);
void make_quartet(int a, int b, int c, int d);
void changedistan(dmatrix distanmat, dvector distanvec, int numspc);
double quartet_lklhd(int a, int b, int c, int d);
double quartet_alklhd(int a, int b, int c, int d);
double usertree_lklhd(FILE *ifp);
double usertree_alklhd(FILE *ifp);
void mlstart(void);
void distupdate(int a, int b, int c, int d);
void mlfinish(void);
void prbranch(Node *up, int depth, int m, int maxm, ivector umbrella, ivector column, FILE *outfp);
void getproportion(double *proportion, dvector distanvec, int maxpair);
void prtopology(FILE *outfp);
void fputphylogeny(FILE *fp);
void resulttree(FILE *outfp);

double gammln(double xx);
void gcf(double *gammcf, double a, double x, double *gln);
void gser(double *gamser, double a, double x, double *gln);
double gammq(double a, double x);
double chi2test(double *ef, int *of, int numcat);
double homogentest(int taxon);
double invgamreg(double a, double x, double pp);
void updaterates(void);
double brent(double ax, double bx, double cx, double (*f )(double ), double tol, double *foptx, double *f2optx, double fax, double fbx, double fcx);
double onedimenmin(double xmin, double xguess, double xmax, double (*f )(double ), double tol, double *fx, double *f2x);
void computestat(double *data, int n, double *mean, double *err);
double quartetml(int a, int b, int c, int d);
double optab(double x);
double optyr(double x);
double optga(double x);
double optla(double x);
void optimquartet(int a, int b, int c, int d);
void optimparams(void);

int gettpmradix(void);
int getcodelength(void);
void rtfdata(dmatrix a, double *f);
int code2int(cvector c);
char *int2code(int s);

void jttdata(dmatrix a, double *f);
void dyhfdata(dmatrix a, double *f);
void mtrevdata(dmatrix r, double *f);

